#include <assert.h> 
#include <stdio.h>
#include <pthread.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

/* Struktura koja sadrzi promenljivu i njen muteks. */
typedef struct _sync_int {
  int v;
  pthread_mutex_t lock;
} sync_int;

sync_int x;

/* Promenljive koje cuvaju dimenziju vektora i same vektore. */
int n;
int v1[100];
int v2[100];

void error_fatal(char *message){
  perror(message);
  exit(EXIT_FAILURE);
}

void pthread_error(char *message, int err){
  fprintf(stderr, "%s : %s\n", message, strerror(err));
  exit (EXIT_FAILURE);
}

void* fun(void *arg){
  int err;
  int i = *((int*)arg);

  /* Zakljucavanje varijable x za promene. */
  if ( (err = pthread_mutex_lock(&x.lock)) != 0)
    pthread_error ("mutex lock error", err);
  x.v += v1[i] * v2[i];
  /* Otkljucavanje varijable x. */
  if ( (err = pthread_mutex_unlock(&x.lock)) != 0)
    pthread_error ("mutex unlock error", err);

  return ((void*)0);
}

/* Citanje dimenzije vektora i samih vektora iz fajla argv[1]. */
void ucitaj_dimenziju_i_vektore (const char *file_name){
  int i;
  FILE *f = fopen(file_name, "r");
  if(f == NULL)
    error_fatal("fopen");
  
  if(fscanf(f, "%d", &n) != 1)
    error_fatal("scanf n");
  
  for(i = 0; i < n; i++)
    if(fscanf(f, "%d", &v1[i]) != 1)
      error_fatal("scanf v1");
    
  for(i = 0; i < n; i++)
    if(fscanf(f, "%d", &v2[i]) != 1)
      error_fatal("scanf v2");
    
  fclose(f);
}

int main(int argc, char **argv)
{
  int i, err;

  if (argc < 2){
    fprintf (stderr, "Fajl sa dimenzijom i vektorima potreban kao argument programa.\n");
    exit (EXIT_FAILURE);
  }
  ucitaj_dimenziju_i_vektore (argv[1]);
  
  /* Alokacija za identifikatore niti */  
  pthread_t* tid_a = (pthread_t*)malloc(n * sizeof(pthread_t));
  if(tid_a == NULL)
    error_fatal("malloc thread ids");
  
  /* Alokacija za argumente niti. */
  int* indices = (int*)malloc(n * sizeof(int));
  if(indices == NULL)
    error_fatal("malloc arguments of the threads");
  
  /* Neophodna je inicijalizacija muteksa pre nego sto ga je moguce koristiti. */
  if( (err = pthread_mutex_init(&x.lock, NULL)) != 0)
    pthread_error ("mutex init error", err);

  printf ("%d\n", n);  
  /* Kreiranje niti. */
  for(i = 0; i < n; i++){
    indices[i] = i;
      
    if( (err = pthread_create(&tid_a[i], NULL, fun, &indices[i])) != 0)
      pthread_error ("thread create error", err);
  }

  /* Cekanje na sve niti. */
  for(i = 0; i < n; i++){
    if( (err = pthread_join(tid_a[i], NULL)) != 0)
      pthread_error ("thread join error", err);
  }
  
  /* Neophodno brisanje muteksa na kraju. */
  if( (err = pthread_mutex_destroy(&x.lock)) != 0)
    pthread_error ("destroy mutex error", err);
  
  /* Oslobadjanje alocirane memorije. */
  free(indices);
  free(tid_a);
  
  printf("%d\n", x.v);
  return 0;
}

